/*
*****************************************************************
*	Restricted VT100 Emulator Developed by			*
*		Michael Grobe					*
*		Vijay Kolli					*
*		User Services					*
*		The University of Kansas			*
*		Last Update	06/09/92			*
*								*
*		This file contains				*
*		SendIntoPort()					*
*		GetCharFromBuffer()				*
*		emulator()					*
*								*
* This Program Process VT100 Escape Sequences and Control       *
* Characters							*
*****************************************************************
*/

/*
The following control characters and Escape sequences have been implemented

Control Characters:
^G		Sound DEC-style beep or flash screen
^H		Backspace, move cursor left one character
^I		Horizontal tab, move cursor to next tabstop
^J		Linefeed, move cursor down one line
^K		Vertical Tab, treated as a linefeed
^L		Formfeed, treated as a linefeed
^M		Carriage return, move cursor to column 1
^O		Locking shift 0, invoke character set in G0 to GL
^X		Cancel escape or control sequence in progress
^Z		Treated the same as above

*****************************************************************

Escape Sequences:

ESC	7	Save cursor position, GL and GR character sets,
		wrap flag, origin mode.
ESC	8	Restore cursor and other information from above
CSI	Pn @	Insert Pn spaces at and after cursor
CSI	Pn A	Cursor up Pn lines, does not scroll
CSI 	Pn B	Cursor down Pn lines, does not scroll
CSI	Pn C	Cursor forward, stays on same line
CSI	Pn D	Cursor backward, stays on same line
CSI 	Pn E	ANSI next line, do Pn times(new line)
CSI	Pn F	Previous-line (reverse index), do Pn times
CSI	Pc G	ANSI cursor to absolute column Pc
CSI	Pr;Pc H	Set cursor to row Pr,column Pc (same as HVP)
CSI	Pn I	Cursor forward Pn tabstops

CSI	Ps J	Erase in display where Ps = 0,1,2
CSI	0  J	Erase from cursor to end of screen, inclusive
CSI	1  J	Erase from start of screen to cursor inclusive
CSI	2  J	Erase entire screen, reset lines to single width,
		cursor does not move

CSI	Ps K	Erase in line
CSI	0  K    Erase from cursor to end of line, inclusive
CSI	1  K 	Erase from start of line to cursor, inclusive
CSI	2  K 	Erase entire line, cursor does not move

CSI	Pn L	Insert Pn lines preceding current line
CSI	Pn M	Delete Pn lines from current downward
CSI	Pn P	Delete Pn chars from cursor to left
CSI	Pn X	Erase Pn chars at and right of cursor
CSI	Pr;Pc f Set cursor to row Pr, column Pc
CSI	Pa;...Pa l 	Reset ANSI mode
CSI	Ps;...Ps m	Select graphic rendition
		0	Attributes 1,4,5 7 off and restore original colors
		1	Bold(intensify foreground)
		4	Underline(reverse video with non-Mono video adapters
		5	Blink
		7	Reverse video, per character

***************************************************************  
*/
#include "mydefs.h"

/* some former global variables 
 * int 	TableLookup[NUM_STATES][NUM_INPUTS];
 * int 	NextState[MAX_TRANSITIONS];
 * int	Action[MAX_TRANSITIONS];
 */
/* sock and end_emulator are still global */
   int norm_sock;
   BOOLEAN end_emulator=FALSE;
   char return_file[256];
/*
*****************************************************************
*								*
*	SendIntoPort()						*
* This routine gets input from keyboard and changes the format	*
* so that it can be sent into the socket.			*
*								*
*****************************************************************
*/

void SendIntoPort(int sig)
{
  int SockChar;
  char Dummy;

#ifdef UNIX
	signal(SIGIO,SIG_IGN); /* */
#endif UNIX

     if(sig==SIGINT) {  /* if the user pressed a control c */
	end_emulator=TRUE;	/*  */
	write(norm_sock,"qQeE\004\003",6); /* send q's and a control c and */
				    /* a controd d to try and terminate */
     } else {

#ifdef VMS
	if(typahead()) {
	    SockChar = mygetch();
	} else
	    return;
#else
	SockChar = mygetch();
#endif VMS

	switch(SockChar) {
	case DNARROW:
#ifdef VMS
			socket_write(norm_sock,"\033[B",3);
#else
			write(norm_sock,"\033[B",3);
#endif VMS
			break;
	case UPARROW:	
#ifdef VMS
			socket_write(norm_sock,"\033[A",3);
#else
			write(norm_sock,"\033[A",3);
#endif VMS
			break;
	case LTARROW:
#ifdef VMS
			socket_write(norm_sock,"\033[D",3);
#else
			write(norm_sock,"\033[D",3);
#endif VMS
			break;
	case RTARROW:
#ifdef VMS
			socket_write(norm_sock,"\033[C",3);
#else
			write(norm_sock,"\033[C",3);
#endif VMS
			break;
	/*
	 * send the address of the character to be
	 * compatible with AIX systems
	 */
	default:
			Dummy = (char) SockChar;
#ifdef VMS
			socket_write(norm_sock,&Dummy,1);
#else
			write(norm_sock,&Dummy,1);
#endif VMS
			break;
	} 
     }
#ifdef UNIX
	signal ( SIGIO, SendIntoPort); /* */
#endif UNIX
}


/*
*****************************************************************
*								*
*		GetCharFromBuffer()				*
* This routine returns a character from the socket.		*
*****************************************************************
*/

int GetCharFromBuffer()
{
	static int CurrentBuf = -1;
	static int CurrentBufPtr = -1;
	static char Buffer[256];

	if (CurrentBuf >= 1) {
		--CurrentBuf;
		++CurrentBufPtr;
		return((int)Buffer[CurrentBufPtr]);
	}
	else
	{
		refresh();
		/*
		 * Need to refresh the screen before getting more
		 * bytes from the socket.
		 */
		/* norm_sock will be -1 if a ctrl-c was pressed */
		if(norm_sock == -1)
		    return(-1);

#ifdef VMS
		if((CurrentBuf = socket_read(norm_sock,Buffer,NUM_BYTES)) <= 0)
		    if(socket_errno != EWOULDBLOCK)
#else 
		if((CurrentBuf = read(norm_sock,Buffer,NUM_BYTES)) <= 0)
#endif VMS
		
		       return(-1);
		CurrentBufPtr = -1;
		--CurrentBuf;
		++CurrentBufPtr;
		return((int)Buffer[CurrentBufPtr]);
	}
}

/*
*****************************************************************
*								*
*			emulator()				*
*****************************************************************
*/

char *emulator(char *host, int port, char *user, char *password, char *command, 
	 BOOLEAN emulate)
{
	int CharInput,State,CurrentAction,index,CharSet=ASCII;
	struct sockaddr_in addr;
	struct hostent *rmt_hostname, *gethostbyname();
	int fd2p=1;  /* sideband error and signal transmission socket */
/* used to be global */
        short 	TableLookup[NUM_STATES][NUM_INPUTS];
        short	NextState[MAX_TRANSITIONS];
        short	Action[MAX_TRANSITIONS];
	int maxfdp1;
	int Enable=1;  /* enable non-blocking i/o */
	int count=0;
	
	/*
	 * If there is anything from the Keyboard then call SendIntoPort
	 */

	*return_file='\0';  /* set to NULL */

	if(emulate==TRUE)
	     initialize(TableLookup,NextState,Action);

#ifdef UNIX
	signal(SIGIO, SendIntoPort ); /*  */ 
#endif UNIX
	signal(SIGINT, SendIntoPort );  

	/* Build the entire address for the remote system. */
	if(port > 0) {  /* connecting to a port */
	  if  ((rmt_hostname = gethostbyname(host)) == NULL)  {
		move(22,0);
		printw("Error mapping host name: %s  to IP address\n", host);
		refresh();
#ifdef UNIX
	        signal(SIGIO,SIG_IGN); /* */
#endif UNIX
		return(return_file);
	  }
	  bcopy(rmt_hostname->h_addr, (char *) &addr.sin_addr.s_addr,
		rmt_hostname->h_length);

					/* Use user-specified port. */
	  addr.sin_port = htons ( port);

	  addr.sin_family = AF_INET;      /* Specify the protocol family. */
					/* Create a socket. */
	  norm_sock = socket(AF_INET, SOCK_STREAM, 0);
	  if (norm_sock < 0) {
		move(22,0);
		printw("Error opening stream socket\n");
		refresh();
#ifdef UNIX
	        signal(SIGIO,SIG_IGN); /* */
#endif UNIX
		return(return_file);
	  }
					/* Connect to the remote system. */
	  if(connect (norm_sock, &addr, sizeof(addr)) < 0) {
		move(22,0);
		printw("Error connecting stream socket\n");
		refresh();
#ifdef UNIX
	        signal(SIGIO,SIG_IGN); /* */
#endif UNIX
		return(return_file);
	  }
	} else {  /* if port equals 0  */
	  struct servent *out;
	  out = getservbyname("exec","tcp");
	  norm_sock = rexec(&host,out->s_port,user,password,command,&fd2p);
	  if(norm_sock==-1)
		return(return_file);
	}

#ifdef UNIX
	fcntl (0, F_SETOWN, getpid() );
	fcntl ( 0, F_SETFL, FASYNC ); 
#endif UNIX
	State = 1;
	CharSet = ASCII;

	end_emulator = FALSE;
	
#ifdef VMS
	if(socket_ioctl(norm_sock, FIONBIO, &Enable)) {
	    socket_perror("Unable to un-block socket");
	    exit(1);
	}
#endif VMS

	while(TRUE) {	

#ifdef VMS
	    if(count >= 100) {
	        SendIntoPort(0); /* non blocking */
		count=0;
	    } else 
		count++;
#endif VMS

	    if((CharInput = GetCharFromBuffer()) != -1 && !end_emulator)  {
	      if(emulate==FALSE) {
		fprintf(stdout,"%c",CharInput);
	      } else {
		index = TableLookup[State][CharInput];
		if (index != -1) {
			State = NextState[index];
			CurrentAction = Action[index];
	         } else {
			State = 1;
			CurrentAction = 0;
	         }
		DoAction(CharInput,CurrentAction,&CharSet);
	      }
	    } else {
	      if(port==0) 
		  write(fd2p,SIGINT,1); /* use the sideband to kill process */
#ifdef UNIX
	      signal(SIGIO,SIG_IGN); /* */
#endif UNIX
	      return(return_file);
	    }
	} /* end while */
		
#ifdef UNIX
	signal(SIGIO,SIG_IGN); /* */
#endif UNIX
	return(return_file);
}

