#include "gopher.h"

PUBLIC char *do_gopher ARGS4(char *,newfile, char *,hightext, 
			     char *,search_string, FILE *,current_fp)
{
     GopherStruct *TmpGopher;

     int port, count, tmp;
     char tmpport[8];
     char path[MAXFNAME];
     char host[80];
     char gopher_info[83];
     char *cp, *cp2, *buf, *buf2;
     BOOLEAN form_submit=FALSE;

     TmpGopher = GSnew();

	/*
	 *  newfile should look like:
	 *	'filetype' '-' 'gopher_path' '@' 'host' ' ' 'port'
	 *         1-1/path/name@ukanhost.cc.edu 70 
	 *
	 *     or  FORM@form_server
	 */

	/* if it doesn't fit the mold, try adding 0-0/ to the front */
	if(newfile[1] != '-' && !isdigit(newfile[0])) {
	    /* use path as temporary string */
	    strcpy(path,newfile);
	    sprintf(newfile,"0-0/%s",path);
	}

        if(!strncmp("FORM",newfile+2,4)) {
	     form_submit = TRUE;
	}

	/* parse file type from newfile */
	switch(newfile[0]) {   /* switch on first letter */
	case '0':  /* file */
	case 'l':  /* lynx file */
     	  	GSsetType(TmpGopher, A_FILE);
		break;
	case '1':  /* dir */		
     		GSsetType(TmpGopher, A_DIRECTORY);   
		break;
	case '2':  /* CSO */
		GSsetType(TmpGopher, A_CSO);
		break;
	case '4':  /* Mac BinHex */
     		GSsetType(TmpGopher, A_MACHEX);   
		break;
	case '5':  /* PC Binary */
     		GSsetType(TmpGopher, A_PCBIN);   
		break;
	case '7':  /* index */
#ifndef TRAVERSAL
     		GSsetType(TmpGopher, A_INDEX);   
		statusline("Enter a search string: ");
		Gophergetstr(search_string);
		if(strlen(search_string) == 0)
#endif TRAVERSAL
		    return("NULL");  /* only if TRAVERSAL */
		break;
	case '8':  /* telnet */
#ifndef TRAVERSAL
		GSsetType(TmpGopher, A_TELNET);
		gopher_telnet(newfile);
#endif TRAVERSAL
		return("NULL");
	case 'T':  /* tn3270 */
#ifndef TRAVERSAL
		GSsetType(TmpGopher, A_TN3270);
		gopher_telnet(newfile);
#endif TRAVERSAL
		return("NULL");
        case '9':  /* UNIX Binary */
     		GSsetType(TmpGopher, A_UNIXBIN);   
		break;
        case 'I':  /* Image type */
#ifndef NON_INTERACTIVE
     		GSsetType(TmpGopher, A_IMAGE);   
		if(display == NULL) {
		    statusline("Can't Display! Would you like to save the image to a file? (y/n)");
		    if(toupper(mygetch()) == 'Y')
     			GSsetType(TmpGopher, A_UNIXBIN);   
		    else
		    	return("NULL");
		} 
#endif NON_INTERACTIVE
		break;
        case 'g':  /* Gif Image type */
     		GSsetType(TmpGopher, A_IMAGE);   
		if(display == NULL) {
		    statusline("Can't Display! Would you like to save the image to a file? (y/n)");
		    if(toupper(mygetch()) == 'Y')
     			GSsetType(TmpGopher, A_UNIXBIN);   
		    else
		    	return("NULL");
		} 
		break;
	default:  /* default is a unknown link type */
		statusline("Gopher type is unknown!!!");
		strcpy(newfile,"#NOT FOUND#");
		return("NULL"); 
		break;
	} /* end switch */

	/* newfile has structure [type]-[path]@[host]   */
	cp = newfile+2;
	/* parse path from newfile */
	for (buf=path; *cp != '@' || *(cp-1) == '\\'; cp++)  {
	    if(*cp != '\\' || (*cp=='\\' && *(cp+1)=='\\')) {
	        *buf = *cp;
	        buf++;
	    }
	}
	*buf = '\0';

	/* parse host from newfile after @ */
	cp++; /* go past the @ */
	for (buf=host; *cp != '\0' && *cp != ' ' ; cp++) {
	    *buf = *cp;
	    buf++;
	}
	*buf = '\0';
	
	/* parse port from newfile after @ and host name */
     if(*cp == ' ') { /* if there is the space for the port */
	cp++; /* go past the space */ 
	for (buf=tmpport; *cp != '\0'; cp++) {
	    *buf = *cp;
	    buf++;
        }
	*buf = '\0';
        port = atoi(tmpport);
	if(port == 0) port=gopher_port;
     } else
	port = gopher_port;

     GSsetPort (TmpGopher,  port);
     GSsetTitle(TmpGopher, hightext);

     simplify_address(path);  /* remove /../ and /./ from path */
     GSsetPath (TmpGopher, path);
     GSsetHost (TmpGopher, host);

     sprintf(gopher_info,"@%s %d",host,port);

     if(GSgetType(TmpGopher) == A_CSO) {
	if(!do_cso(TmpGopher))
	    return("NULL");
	statusline("Searching CSO database -- Please Wait");
     }

     if(GSgetType(TmpGopher) == A_MACHEX ||
        GSgetType(TmpGopher) == A_PCBIN ||
        GSgetType(TmpGopher) == A_IMAGE ||
        GSgetType(TmpGopher) == A_UNIXBIN) {
           get_bin_file(TmpGopher, hightext);
	   return("NULL");
     }

     /* make the name the file will become */
     tempname(newfile, NEW_FILE);

	if(GSgetType(TmpGopher) == A_INDEX) 
	    statusline("One moment Please -- Searching Data");
	else {
	    char tempstring[100];
	    sprintf(tempstring,"One moment Please -- Retrieving remote data from %s.",host);
	    statusline(tempstring);
	}

     if(get_file(TmpGopher, newfile, gopher_info, search_string,
					    form_submit, current_fp)==-1) {
	  /* could not get file */
	if(GSgetType(TmpGopher) == A_FILE) {
		/* The hytelnet database files are all messed up
		 * so we have to do this ugly hack here */
		/* first try to uncapitalize the filename */
	    if((cp = strrchr(GSgetPath(TmpGopher),'/')) != NULL) {
		uncapitalize(cp+1);
		if(get_file(TmpGopher,newfile,gopher_info,search_string, 
						   form_submit, NULL)==-1){ 
		/* still couldn't find the file. lets try again with
		 * the first three letters of the file name as a directory 
		 */

	          cp = strrchr(GSgetPath(TmpGopher),'/');
	  	  mystrncpy(path,GSgetPath(TmpGopher),cp-GSgetPath(TmpGopher)+1);
		  strncat(path,cp+1,3);
		  strcat(path,cp);
		  GSsetPath(TmpGopher,path);
		  if(get_file(TmpGopher,newfile,gopher_info,search_string,
							form_submit, NULL)==-1) 
		    strcpy(newfile,"#NOT FOUND#");
		}
	    } else
		strcpy(newfile,"#NOT FOUND#");
	 } else
	    strcpy(newfile,"#NOT FOUND#");
     }

     refresh();  /* paint the screen */

     GSdestroy(TmpGopher);

     return(gopher_info);


} /* big end */	 
