/*
 * This is a crude attempt at doing some object-based programming in C
 */
#if !defined(NeXT)
#ifndef VMS
#ifndef MSDOS
#include <malloc.h>
#endif
#endif
#endif

#include "gopher.h"
int DEBUG_VAR;

/*
 * Make a new gopherstruct...  Should reuse destroyed GopherObjs...
 */

GopherObj *
GSnew()
{
     GopherObj *temp;

     temp = (GopherObj *) malloc(sizeof(GopherObj));
     temp->Selstr = STRnew();
     temp->Title  = STRnew();
     temp->Host   = STRnew();

     GSinit(temp);
     
     return(temp);
}

void
GSdestroy(gs)
  GopherObj *gs;
{
     STRdestroy(gs->Selstr);
     STRdestroy(gs->Title);
     STRdestroy(gs->Host);

     free(gs);
}
     


/*
 * Clear out all the crud 
 */

void
GSinit(gs)
  GopherObj *gs;
{
     GSsetType(gs, '\0');
     
     STRinit(gs->Title);
     STRinit(gs->Selstr);
     STRinit(gs->Host);

     gs->iPort = 0;
     gs->iItemnum = 0;
}


void
GStoNet(gs, sockfd)
  GopherObj *gs;
  int sockfd;
{
     static char buf[1024];

     buf[0] = GSgetType(gs);

     sprintf(buf + 1, "%s\t%s\t%s\t%d\r\n",
	     GSgetTitle(gs),
	     GSgetPath(gs),
	     GSgetHost(gs),
	     GSgetPort(gs));

     writestring(sockfd, buf);
     
     if (DEBUG_VAR)
	  fprintf(stderr, buf);

}

extern int readfield();
extern int readline();

int
GSfromNet(gs, sockfd)
  GopherObj *gs;
  int sockfd;
{
     char *cPtr = NULL;
     char foo[1024];
     
     if (readfield(sockfd, foo, 1024)<= 0) {
	  /* EOF or error */
	  return(1);
     }

     GSsetType(gs, foo[0]);

     /** Get the kind of file from the first character **/
     /** Filter out files that we can't deal with **/

     switch (GSgetType(gs)) {
       case A_FILE:
       case A_DIRECTORY:
       case A_MACHEX:
       case A_CSO:
       case A_INDEX:
       case A_TELNET:
       case A_SOUND:
       case A_MIME:
/*     case A_EVENT:
       case A_CALENDAR:   not ready for prime time yet */
	  break;
       case A_EOI:
	  return(1);
     default:
	return(-1);  
     }

     /** Suck off the User Displayable name **/
     GSsetTitle(gs, foo+1);
     
     /** Suck off the Pathname **/
     if (readfield(sockfd, foo, 1024) == 0)
	  return(-1);
     GSsetPath(gs, foo);

     /** Suck off the hostname **/
     if (readfield(sockfd, foo, 1024) == 0)
	  return(-1);
     GSsetHost(gs, foo);

     if (readline(sockfd, foo, 1024)==0)
	  return(-1); 

     GSsetPort(gs, 0);

     /** Get the port number **/
     GSsetPort(gs, atoi(foo));

     return(0);
}

/** Copy a GopherObj ***/

void
GScpy(dest, orig)
  GopherObj *dest, *orig;
{
     dest->sFileType = orig->sFileType;
     dest->iPort     = orig->iPort;
     dest->iItemnum  = orig->iItemnum;

     GSsetTitle(dest, GSgetTitle(orig));
     GSsetPath(dest, GSgetPath(orig));
     GSsetHost(dest, GSgetHost(orig));
}

/** Compare two GopherObjs ***/

int
GScmp(gs1, gs2)
  GopherObj *gs1, *gs2;
{
     if (GSgetTitle(gs1) == NULL)
	  return(1);
     if (GSgetTitle(gs2) == NULL)
	  return(-1);

     return(strcmp(GSgetTitle(gs1), GSgetTitle(gs2)));
}

/*
 * Really weird!!!  We need this for qsort,  don't know why we can't use
 * GScmp...
 */

int
GSmoocmp(gs1, gs2)
  GopherObj **gs1, **gs2;
{
     if (GSgetTitle(*gs1) == NULL)
	  return(1);
     if (GSgetTitle(*gs2) == NULL)
	  return(-1);

     return(strcmp(GSgetTitle(*gs1), GSgetTitle(*gs2)));
}



/***********************************************************************
** Stuff for GopherDirObjs
**
***********************************************************************/


GopherDirObj*
GDnew(size)
  int size;
{
     int i;
     GopherDirObj *temp;
     
     temp = (GopherDirObj*) malloc(sizeof(GopherDirObj));
     temp->Gophers = (GopherObj **) malloc(size * sizeof(GopherObj*));
     
     for (i = 0; i < size; i++)
	  temp->Gophers[i] = GSnew();

     temp->maxsize = size;

     GDinit(temp);
     return(temp);
}


void
GDdestroy(gd)
  GopherDirObj *gd;
{
     int i;
     
     for (i = 0; i < gd->maxsize; i++)
	  GSdestroy(gd->Gophers[i]);

     free(gd->Gophers);
     free(gd);
}


void
GDinit(gd)
  GopherDirObj *gd;
{
     int i;

     for (i=0; i<gd->maxsize; i++) {
	  GSinit(GDgetEntry(gd, i));
     }

     GDsetTop(gd, 0);
}


/*
 * Increase the size of the GopherObj pointer array to "size"
 */

void
GDgrow(gd, size)
  GopherDirObj *gd;
  int size;
{
     GopherObj **temp;
     int i;

     if (size < gd->maxsize)
	  return; /** Size is smaller than requested **/

     temp = (GopherObj **) realloc(gd->Gophers, size*sizeof(GopherObj*));
 
     if (temp == NULL)
	  fprintf(stderr, "Out of memory!!!\n"), exit(-1);

     if (temp != gd->Gophers) {
	  free(gd->Gophers);
	  gd->Gophers = temp;
     }

     /** Initialize the new GopherObjs.  **/

     for (i= gd->maxsize; i< size; i++)
	  gd->Gophers[i] = GSnew();
     
     gd->maxsize = size;
     return;
}


extern int DEBUG_VAR;

/** This proc adds a GopherObj to a gopherdir. **/
void
GDaddGS(gd, gs)
  GopherDirObj *gd;
  GopherObj *gs;
{
	int n, i;

	if (GDgetTop(gd) == gd->maxsize)
		  GDgrow(gd, gd->maxsize*2);

	if (DEBUG_VAR)
		fprintf(stderr, "Adding %s, Top=%d, Num=%d\n",
			GSgetTitle(gs), GDgetTop(gd), GSgetNum(gs));

	if ((n = GSgetNum(gs)) == 0)	/* if we don't care where it goes */
		n=1;			/* put it in the first available slot */

	if (GSgetType(GDgetEntry(gd, n-1)) != '\0') {	/* that slot is in use*/
		for (i=1; ; i++)		/* find any empty slot */
			if (GSgetType(GDgetEntry(gd, i-1)) == '\0')
				break;		/* got one */
		if (i >= GDgetTop(gd))	/* item was past end of list */
			GDsetTop(gd, i);	/* update end of list */
		if (GSgetNum(gs) != 0) {	/* if we want that slot */
			/* kick out the entry that's in our way */
			GScpy(GDgetEntry(gd, i-1), GDgetEntry(gd, n-1));
		} else {
			n = i;	/* don't be a bully */
		}
	}

	GScpy(GDgetEntry(gd, n-1), gs);		/* put our entry in place */
	if (n >= GDgetTop(gd))			/* item was past end of list */
		GDsetTop(gd, n);		/* update end of list */

}


#ifdef GOPHER_SERVER

void
GDsort(gd)
  GopherDirObj *gd;
{
     int i;

     /*** Find first non-numbered entry ***/

     for (i=0; ; i++) {
	  if (GSgetNum(GDgetEntry(gd, i)) == 0)
	       break;
     }

     /*** Everything up to i is already sorted by user-defined ordering ***/

     if (GDgetTop(gd) <= i)
	  /** No more sorting needed ***/
	  return;

     
     qsort((char *) (&(gd->Gophers[i])), gd->Top-i, 
	   sizeof(GopherObj*),GSmoocmp);

}

#endif


void
GDtoNet(gd, sockfd)
  GopherDirObj *gd;
  int sockfd;
{
     int i;

     for (i=0; i< GDgetTop(gd); i++) {
	  GStoNet(GDgetEntry(gd, i), sockfd);
     }	  

}

int
GDfromNet(gd, sockfd)
  GopherDirObj *gd;
  int sockfd;
{
     static GopherObj *TempGopher;
     static char ZesTmp[1024];
     int j, i;

     if (TempGopher == NULL)
	  TempGopher = GSnew();

     for (j=0; ; j++) {

	  if (j == gd->maxsize)
	       GDgrow(gd, gd->maxsize*2);

	  ZesTmp[0] = '\0';
	  
	  GSinit(TempGopher);
	  i = GSfromNet(TempGopher, sockfd);
	  
	  if (i==0)
	       GDaddGS(gd, TempGopher);

	  if (i==1)
	       return(j);

	  if (i<0) {
	       j = j-1;
	       if (j<0) j=0;
	       readline(sockfd, ZesTmp, 1024);
	  }
     }
} 
